import {
  Controller,
  Get,
  Header,
  Param,
  ParseIntPipe,
  Res,
  StreamableFile,
} from '@nestjs/common';
import { AppService } from './app.service';
import { ApiOperation, ApiParam, ApiProduces } from '@nestjs/swagger';
import { BlankReturnMessageDto } from './dto/ReturnMessage.dto';
import { ConfigService } from '@nestjs/config';
import { Response } from 'express';

@Controller()
export class AppController {
  private readonly urlGetPrefix =
    this.config.get<string>('URL_GET_PREFIX') || '';
  constructor(
    private readonly appService: AppService,
    private config: ConfigService,
  ) {}

  @Get('avatar/:username/:size/:filename.png')
  @ApiOperation({ summary: '获取裁剪后的用户头像' })
  @ApiParam({ name: 'username', description: '用户名' })
  @ApiParam({ name: 'size', description: '头像尺寸', type: 'number' })
  @ApiParam({ name: 'filename', description: '头像文件名' })
  @ApiProduces('image/png')
  async getImage(
    @Param('username') username: string,
    @Param('size', ParseIntPipe) size: number,
    @Res({ passthrough: true }) res: Response,
  ) {
    if (size < 8 || size > 1000) {
      throw new BlankReturnMessageDto(400, 'Invalid size').toException();
    }
    const buffer = await this.appService.getImage(username, size);
    res.header('Cache-Control', 'public, max-age=600');
    return new StreamableFile(buffer, {
      type: 'image/png',
      disposition: 'inline',
    });
  }

  @Get('url/:url/:size/:filename.png')
  @Header('Cache-Control', 'public, max-age=31536000')
  @ApiOperation({ summary: '获取裁剪后的用户头像' })
  @ApiParam({ name: 'url', description: '地址，没有前缀' })
  @ApiParam({ name: 'size', description: '头像尺寸', type: 'number' })
  @ApiParam({ name: 'filename', description: '图片文件名' })
  @ApiProduces('image/png')
  async getAnyImage(
    @Param('url') url: string,
    @Param('size', ParseIntPipe) size: number,
    @Res({ passthrough: true }) res: Response,
  ) {
    if (size < 8 || size > 1000) {
      throw new BlankReturnMessageDto(400, 'Invalid size').toException();
    }
    if (url.length > 255 - this.urlGetPrefix.length) {
      throw new BlankReturnMessageDto(400, 'Invalid url').toException();
    }
    const buffer = await this.appService.getResizedImage(
      `${this.urlGetPrefix}${url}`,
      size,
    );
    res.header('Cache-Control', 'public, max-age=31536000');
    return new StreamableFile(buffer, {
      type: 'image/png',
      disposition: 'inline',
    });
  }
}
